/*
 *	USt Dashboard Widget
 *	
 *	Copyright 2005 Max Antoni
 *	http://www.maxantoni.de
 */
var canvasWidth = 145;
var canvasHeight = 90;
var canvasOffsetLeft = 7;
var canvasOffsetTop = 5;
var edgeRadius = 12;

var URL_HOMEPAGE = "http://www.maxantoni.de";
var BUTTON_DONE = "Done";
var DOT = "dot";
var DOT_DEFAULT = ".";
var UST_DEFAULT = "default";
var UST_DEFAULT_VALUE = "17" + DOT_DEFAULT + "5";

var dot = getLocalizedString(DOT, DOT_DEFAULT);

var factor = Number(getPref("ustSatz", getLocalizedString(UST_DEFAULT, UST_DEFAULT_VALUE))) / 100;

var kanne = new Image();
kanne.src = "kanne.png";

function initWidget() {
	createGenericButton(doneButton, getLocalizedString(BUTTON_DONE), showFront);
	draw(canvasFront.getContext("2d"), "#aa00aa", "#550055", "#330033");
    draw(canvasBack.getContext("2d"), "#aaaaaa", "#555555", "#333333");
}

function drawFront() {
    back.style.visibility = "hidden";
    front.style.visibility = "visible";
	if(ustDisplay.hasChildNodes())
		ustDisplay.removeChild(ustDisplay.firstChild);
	var v = getPref("ustSatz", getLocalizedString(UST_DEFAULT, UST_DEFAULT_VALUE));
	v = v.replace(".", dot);
    ustDisplay.appendChild(document.createTextNode(v));
	netto.value = "";
	ust.value = "";
	brutto.value = "";
	netto.focus();
}

function drawBack() {
    front.style.visibility = "hidden";
    back.style.visibility = "visible";
}

function draw(context, color1, color2, colorBorder) {
	context.clearRect(0, 0, canvasWidth + canvasOffsetLeft * 2,
						canvasHeight + canvasOffsetTop * 2 + canvasHeight);

	createPath(context);
	context.save();
	context.setFillColor(0.0, 0.0, 0.0, 1.0);
	context.setShadow(0, 7, 10, 0.0, 0.5);
	context.fill();
	context.restore();
	
	context.save();
	createPath(context);
	context.fillStyle = context.createLinearGradient(0,0,0,canvasHeight);
	context.fillStyle.addColorStop(0, "#ffffff");
	context.fillStyle.addColorStop(0.2, color1);
	context.fillStyle.addColorStop(1, color2);
	context.fill();
	context.restore();

	context.save();
	createPath(context);
	context.setStrokeColor(colorBorder, 1.0);
	context.setLineWidth(1.5);
	context.setLineCap("round");
	context.stroke();
	context.restore();
	
	context.save();
	createPath(context);
	context.clip();
	context.drawImage(kanne, 5, 0);
	context.restore();
}

function createPath(context) {
	context.beginPath();
	context.moveTo(canvasOffsetLeft + edgeRadius, canvasOffsetTop);
	context.arcTo(canvasOffsetLeft + canvasWidth, canvasOffsetTop,
					canvasOffsetLeft + canvasWidth, canvasOffsetTop + edgeRadius,
					edgeRadius);
	context.arcTo(canvasOffsetLeft + canvasWidth, canvasOffsetTop + canvasHeight,
					canvasOffsetLeft + canvasWidth - edgeRadius, canvasOffsetTop + canvasHeight,
					edgeRadius);
	context.arcTo(canvasOffsetLeft, canvasOffsetTop + canvasHeight,
					canvasOffsetLeft, canvasOffsetTop + canvasHeight - edgeRadius,
					edgeRadius);
	context.arcTo(canvasOffsetLeft, canvasOffsetTop,
					canvasOffsetLeft + edgeRadius, canvasOffsetTop,
					edgeRadius);
	context.closePath();
}

function toDisplay(n) {
	var o = String(Math.round(n * 100) / 100);
	var i = o.indexOf(".");
	if(i < 0)
		return o + dot + "00";
	o = o.substring(0, i) + dot + o.substring(i + 1);
	if(o.length - i == 2)
		return o + "0";
	return o;
}

function calculate() {
	var o = netto.value;
	if(o.length == 0) {
		ust.value = "";
		brutto.value = "";
		return;
	}
	var n = asNumber(o);
	if(n) {
		var u = n * factor;
		ust.value = toDisplay(u);
		brutto.value = toDisplay(n + u);
	}
	else {
		ust.value = "?";
		brutto.value = "?";
	}
}

function calculateReverse() {
	var o = brutto.value;
	if(o.length == 0) {
		ust.value = "";
		netto.value = "";
		return;
	}
	var b = asNumber(o);
	if(b) {
		var n = b / (1+factor);
		ust.value = toDisplay(b - n);
		netto.value = toDisplay(n);
	}
	else {
		ust.value = "?";
		netto.value = "?";
	}
}

function asNumber(o) {
	var i = o.indexOf(dot);
	return Number(i < 0 ? o : o.substring(0, i) + "." + o.substring(i + 1));
}

function showMaxAntoni() {
	if(window.widget) {
		widget.openURL(getLocalizedString(URL_HOMEPAGE));
	}
	else {
		alert(getLocalizedString(URL_HOMEPAGE));
	}
}

function showBack() {
	exitflip();
    if(window.widget)
        widget.prepareForTransition("ToBack");       
    drawBack();
    if(window.widget)
        setTimeout('widget.performTransition();', 0);
    ustSatz.value = getPref("ustSatz", getLocalizedString(UST_DEFAULT, UST_DEFAULT_VALUE)).replace(".", dot);
    ustSatz.focus();
}

function showFront() {
	var n = asNumber(ustSatz.value);
	if(n) {
		if(factor != n / 100) {
			factor = n / 100;
			setPref("ustSatz", String(n));
		}
	}
    if(window.widget)
        widget.prepareForTransition("ToFront");
    drawFront();
    if(window.widget)
        setTimeout('widget.performTransition();', 0);
}

function checkNumber(event) {
	if(event.metaKey) {
		return;
	}
	if(event.keyCode == 13) {
		if(event.target == ustSatz) {
			showFront();
			return;
		}
		netto.value = "";
		ust.value = "";
		brutto.value = "";
		event.target.focus();
		return;
	}
	if(event.keyCode == 0 || event.keyCode == 9) {
		event.preventDefault();
		if(event.target == netto)
			brutto.focus();
		else if(event.target == brutto)
			netto.focus();
		return;
	}
	//	alert(event.keyCode + " " + dot);
	if(event.keyCode == 8 || event.keyCode == 37 || event.keyCode == 39 || event.keyCode == 46)
		return
	if(event.keyCode == 110 || event.keyCode == 188) {
		if("." == dot) event.preventDefault();
		return;
	}
	if(event.keyCode == 190) {
		if("," == dot) event.preventDefault();
		return;
	}
	if((event.keyCode < 48 || event.keyCode > 57) && (event.keyCode < 96 || event.keyCode > 105))
		event.preventDefault();
}

function checkRange(element, min, max) {
	if(element.value == "")
		return;
	if(Number(element.value) < min)
		element.value = String(min);
	else if(Number(element.value) > max)
		element.value = String(max);
}

function getPref(key, def) {
	if(window.widget) {
    	var value = widget.preferenceForKey(widget.identifier + "_" + key);
    	if(value) {
        	return value;
    	}
	}
	return def;
}

function setPref(key, value) {
	if(window.widget) {
		widget.setPreferenceForKey(value, widget.identifier + "_" + key);
	}
}

function getLocalizedString(key, def) {
	try {
    	var value = localizedStrings[key];
    	if(value === undefined)
    		return def === undefined ? key : def;
		return value;
	}
	catch(e) {
    	return def === undefined ? key : def;
    }
}
